#include <grape.h>


extern COMPLEX CZERO;
extern COMPLEX CE;
extern COMPLEX cmplx(double, double);
extern COMPLEX cadd(COMPLEX, COMPLEX);
extern COMPLEX cinv(COMPLEX);
extern COMPLEX crmul(COMPLEX, double);
extern COMPLEX cpower(COMPLEX, int);
extern COMPLEX cmul(COMPLEX, COMPLEX);


COMPLEX my_get_arg(AMANDUS *amandus, int part, double u, double v)
{
    if(amandus->class != Amandus)
	return cmplx(u, v);

    if(!strcmp(amandus->name, "my_cat_enn"))
        return cmplx(u*cos(v), u*sin(v));

    ASSURE(0, "my_get_arg: unknown surface", return(CZERO));
}


EXPLICIT *explicit_set_compute_arg()
{
    EXPLICIT *explicit;

    explicit = (EXPLICIT *)START_METHOD(G_INSTANCE);
    ASSURE(explicit, "", END_METHOD(NULL));

    explicit->compute_arg = my_get_arg;

    END_METHOD(explicit);
}


/*
 *   my_cat_enn, copy of cat_enn for testing
 *
 *   g  = 1 / z + eps * z^k
 *   f  = 1
 *   dg = -1 / z^2 + eps k z^(k-1)
 */

static int *my_cat_enn_k;
static double *my_cat_enn_eps;

static COMPLEX my_cat_enn_g(COMPLEX z)
{
    return (cadd (cinv (z), crmul (cpower (z, *my_cat_enn_k),
				   *my_cat_enn_eps)));
}

static COMPLEX my_cat_enn_f(COMPLEX z)
{
    return(CE);
}

static COMPLEX my_cat_enn_dg(COMPLEX z)
{
    return cadd(crmul(cpower(z, *my_cat_enn_k-1),
                      *my_cat_enn_eps*(double)*my_cat_enn_k),
                crmul(cinv(cmul(z, z)), -1.0));
}


AMANDUS *amandus_set_weier_f()
{
    AMANDUS *amandus;

    amandus = (AMANDUS *)START_METHOD(G_INSTANCE);
    ASSURE(amandus, "", END_METHOD(NULL));

    if(!strcmp(amandus->name, "my_cat_enn")) {
	my_cat_enn_k   = &amandus->time_parm->int_parm[0];
	my_cat_enn_eps = &amandus->time_parm->double_parm[0];
	amandus->weier_f.g  = my_cat_enn_g;
	amandus->weier_f.dh = my_cat_enn_f;
	amandus->weier_f.dg = my_cat_enn_dg;
    }	
		
    END_METHOD(amandus);
}
