/* grape-methode "compute"
 *
 * Call              : GRAPE(time_curve, "compute-send")()
 *
 * Parameters        : TIME_CURVE *time_curve
 *
 * Description       : Compute the time curve using the curve_parm
 *                     parameters and functions. The points at which
 *                     the functions are evaluated are controlled by
 *                     the curve_parm->dstrbn spline.
 *
 * Author            : Konrad Polthier
 *
 * Copyright (c) 1992  by Universitaet Bonn
 *                        Institut fuer Angewandte Mathematik
 *                        Sonderforschungsbereich 256
 *                        D-53115 Bonn
 */

#include <grape.h>
#include "curve.h"


double dstrbn(int, int, double, double, SPLINE1D *, LIST_OF_DOUBLE *);


TIME_CURVE *time_curve_compute_send()
{
  int i;
  double arg[10];
  CURVE_PARM *parm;
  TIME_CURVE *self;
  TRIANG1D *curve;

  self = (TIME_CURVE *)START_METHOD(G_INSTANCE);
  ASSURE(self, "", END_METHOD(NULL));

  ASSURE(self->geometry && self->geometry->object,
	 "time_curve_compute_send: no geometry for compute",
	 END_METHOD(NULL));

  ASSURE(self->coord_f.x && self->coord_f.y && self->coord_f.z,
	 "time_curve_compute_send: coordinate function missing",
	 END_METHOD(NULL));

  /* get post-object from the timestep with time <= self->time */
  curve = (TRIANG1D *)
    GRAPE(self->geometry->object, "get-post-obj")(*self->time);

  /*
   * assign arguments for fn_eval call to array arg - arguments are
   *    arg[0]   - where to evaluate the function
   *    arg[1]   - time
   *    arg[2-8] - function parameters (int/double converted to double)
   *
   * arg[0] will be computed from the distribution function below,
   * arg[1-8] will be handled by the time_object function
   */
  g_time_parm_get_var_values(self->time_parm, *self->time, &(arg[1]));

  parm = self->curve_parm;

  /*
   * the simplest way to evaluate the function at parm->num_of_points is
   *
   *    delta = (parm->max - parm->min) / (parm->num_of_points - 1);
   *    arg[0] = parm->min;
   *    for(i = 0; i < parm->num_of_points, i++) {
   *       ...
   *	     arg[0] += delta;
   *    }
   *
   * instead of these equal distant points a spline is used which allows
   * the user to freely distribute the points in the domain interval,
   * by default (no user defined spline) a linear spline is used
   */
  for(i = 0; i < parm->num_of_points; i++) {
    if(parm->min < parm->max)
      arg[0] = dstrbn(i, parm->num_of_points - 1,
		      parm->min, parm->max, parm->dstrbn, NULL);
    else
      arg[0] = dstrbn (parm->num_of_points - 1 - i,
		       parm->num_of_points - 1,
		       parm->max, parm->min, parm->dstrbn, NULL);
    curve->x[i] = fn_eval(self->coord_f.x, arg);
    curve->y[i] = fn_eval(self->coord_f.y, arg);
    curve->z[i] = fn_eval(self->coord_f.z, arg);
  }
  curve->number_of_points = parm->num_of_points;

  END_METHOD(self);
}
