/* grape-methode "get-object"
 *
 * Call              : GRAPE(time_curve, "get-object")(inst, time);
 *
 * Parameters        : TIME_CURVE *time_curve
 *                     INSTANCE *inst
 *                     double time
 *
 * Return value      : INSTANCE *inst
 *
 * Description       : Create the time_curve->inter instance for the
 *                     given time and assign it to inst if necessary.
 *                     See section "What is a dynamic object" in the
 *                     documentation for a further explanation.
 *
 * Author            : Konrad Polthier
 *
 * Copyright (c) 1992  by Universitaet Bonn
 *                        Institut fuer Angewandte Mathematik
 *                        Sonderforschungsbereich 256
 *                        D-53115 Bonn
 */

#include <grape.h>
#include "curve.h"


INSTANCE *time_curve_get_object(INSTANCE *inst, double time)
{
  TIME_CURVE *self;
  double time_sv;

  self = (TIME_CURVE *)START_METHOD(G_INSTANCE);
  ASSURE(self, "", END_METHOD(NULL));

  ASSURE(self->geometry,
	 "time_curve_get_object: no time_curve->geometry",
	 END_METHOD(NULL));

  /* correct time if it's not in [0, 1] */
  time_sv = time;
  g_check_bounds(&time, 0.0, 1.0);

  if(fabs(time - time_sv) > ZERO_MACHINE)
    fprintf(stderr, "time_curve_get_object: time adjusted to [0, 1]\n");

  /*
   * get time-section for the given time from the geometry and
   * assign it to the time_curve's interpol instance
   */
  self->inter = (CHAIN *)
    GRAPE(self->geometry, "get-object")(self->inter, time);

  /* if given instance is not time_curve's inter assign it to the inst */
  if(inst != (INSTANCE *)self->inter)
    ASSIGN(inst, (INSTANCE *)self->inter);

  END_METHOD(inst);
}
