/* grape-methods "curve-setup"
 *               "curve-add"
 *
 * Call              : GRAPE(project, "curve-setup")();
 *                     GRAPE(project, "curve-add")();
 *
 * Parameters        : PROJECT *project
 *
 * Description       : "curve-setup" is the initial setup routine for
 *                     project (Time_)Curve. It assigns all classes,
 *                     methods, buttons etc which belong to this project.
 *                     It also declares which (-init, -exit and) -add
 *                     methods should be used.
 *
 *                     "curve-add" is called whenever this project is added
 *                     via "add" e.g. when the button "Project: add" in
 *                     the main menu of the manager is pressed. This
 *                     method creates the hierarchy to be included in the
 *                     managers hierarchy and sets the curve version
 *                     message.
 *
 *                     See section "How to configure a project" in the
 *                     documentation for further explanation.
 *
 * Author            : Bernd Oberknapp, Konrad Polthier,
 *
 * Copyright (c) 1992  by Universitaet Bonn
 *                        Institut fuer Angewandte Mathematik
 *                        Sonderforschungsbereich 256
 *                        D-53115 Bonn
 */

#include <grape.h>
#include "curve.h"
#include <string.h>


/* classes declared in project Time_Curve */
CLASS *Time_Curve = NULL, *Curve_Parm = NULL;


/* methods declared in project Time_Curve */
PROJECT *project_curve_add();

void *curve_parm_delete();
void *curve_parm_list_alloc();
void *curve_parm_list_free();

void *time_curve_new_instance();
void *time_curve_compute_send();
void *time_curve_delete();
void *time_curve_get_object();
void *time_curve_list_alloc();
void *time_curve_list_free();
void *time_curve_get_extension();
void *time_curve_read_data();
void *time_curve_get_info();


/* list of new classes to be added (null terminated) */
static CLASS_DESCR curve_class_descr[] = {
  { &Time_Object, &Time_Curve, "Time_Curve", sizeof(TIME_CURVE) },
  { &Root,        &Curve_Parm, "Curve_Parm", sizeof(CURVE_PARM) },

  { NULL, NULL, NULL, 0 }
};


/* list of new methods to be added (null terminated) */
static METHOD_DESCR curve_method_descr[] = {
  /* methods for project management */
  { &Project, "curve-add", (void *(*)())project_curve_add },

  /* project methods */
  { &Curve_Parm, "delete",     curve_parm_delete     },
  { &Curve_Parm, "list-alloc", curve_parm_list_alloc },
  { &Curve_Parm, "list-free",  curve_parm_list_free  },

  { &Time_Curve, "new-instance",  time_curve_new_instance  },
  { &Time_Curve, "compute-send",  time_curve_compute_send  },
  { &Time_Curve, "delete",        time_curve_delete        },
  { &Time_Curve, "list-alloc",    time_curve_list_alloc    },
  { &Time_Curve, "list-free",     time_curve_list_free     },
  { &Time_Curve, "get-extension", time_curve_get_extension },
  { &Time_Curve, "read-data",     time_curve_read_data     },
  { &Time_Curve, "get-object",    time_curve_get_object    },
  { &Time_Curve, "get-info",      time_curve_get_info      },

  { NULL, NULL, NULL }
};


PROJECT *project_curve_setup()
{
  PROJECT *self;

  self = (PROJECT *)START_METHOD(G_INSTANCE);
  ASSURE(self, "", END_METHOD(NULL));

  /*
   * assign all classes, methods and buttons to project,
   * declare that we depend on project time_object
   */
  GRAPE(self, "setup")(P_Project, "time_object",
		       P_ClassList, curve_class_descr,
		       P_MethodList, curve_method_descr,
		       P_AddMethod, "curve-add",
		       P_Version, 3, 1,
		       P_End);

  END_METHOD(self);
}


PROJECT *project_curve_add()
{
  PROJECT *self;
  TIME_CURVE *tc;

  self = (PROJECT *)START_METHOD(G_INSTANCE);
  ASSURE(self, "", END_METHOD(NULL));

  /* create Time_Curve instance and link it in the hierarchy */
  tc = (TIME_CURVE *)GRAPE(Time_Curve, "new-instance")("curve");
  GRAPE(self, "set-object")(tc);

  END_METHOD(self);
}
